package com.totopia.server.modules.user.entity;

import java.util.Date;
import java.util.Set;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.UniqueConstraint;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.totopia.server.commons.data.entity.DateAuditEntity;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.Builder.Default;
import lombok.experimental.SuperBuilder;

@Entity
@Table(name = "users", uniqueConstraints = { @UniqueConstraint(columnNames = { "username" }),
    @UniqueConstraint(columnNames = { "email" }) })
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = false)
public class UserEntity extends DateAuditEntity {
  private static final long serialVersionUID = 8891163223262220481L;

  @Id
  @Column(name = "username", unique = true, nullable = false, length = 150)
  private String username;

  @Basic
  @Column(name = "password", nullable = false, length = 100)
  @JsonIgnore
  private String password;

  @Basic
  @Column(name = "nickname", nullable = false, length = 150)
  private String nickname;

  @Basic
  @Column(name = "email", nullable = false, length = 100)
  private String email;

  @Basic
  @Column(name = "block", nullable = false)
  @Default
  private Boolean block = false;

  @Basic
  @Column(name = "send_email", nullable = false)
  @Default
  private Boolean sendEmail = true;

  @Basic
  @Column(name = "activation", nullable = true, length = 100)
  private String activation;

  @Basic
  @Temporal(TemporalType.TIMESTAMP)
  @Column(name = "last_reset_time", nullable = true)
  private Date lastResetTime;

  @Basic
  @Column(name = "reset_count", nullable = false)
  @Default
  private Long resetCount = 0L;

  @Basic
  @Column(name = "otp_key", nullable = true, length = 1000)
  private String otpKey;

  @Basic
  @Column(name = "otep", nullable = true, length = 1000)
  private String otep;

  @Basic
  @Column(name = "require_reset", nullable = true)
  @Default
  private Boolean requireReset = false;

  @ManyToMany(fetch = FetchType.EAGER)
  @JoinTable(name = "user_roles", joinColumns = @JoinColumn(name = "user_id"), inverseJoinColumns = @JoinColumn(name = "role_id"))
  private Set<RoleEntity> roles;
}

// 아이디
// 로그인 아이디
// 로그인 패스워드
// 로그인 패스워드 문자
// 이메일
// 닉네임
// 은행명
// 계좌번호
// 예금주
// 추천인
// 추천수
// 게시판 제한 여부
// 쿠폰
// 충전방식
// 비고
// 종목별 베팅제한
// 상태표기
// 휴대폰번호
// 추천권한 여부
// 가입상태
// 소속
// 레벨
// 보유머니
// 포인트
// 회원상태
// 룰렛개수
// 계좌순번
// 가입날짜
// 최근접속 날짜
// 가입 아이피
// 최근 접속 아이피
// 베팅 알림
// API 연결
